#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest
from duckhunt import duckhunt
from datetime import datetime
import pickle
import json


@duckhunt
class Passenger(object):
    a_str = str
    a_int = int
    a_list = list
    a_dict = dict
    a_tuple = tuple
    a_object = datetime


@duckhunt
class Bom(object):
    a_str = str
    a_int = int
    a_list = list
    a_dict = dict
    a_tuple = tuple
    a_object = datetime
    a_duckhunt = Passenger


class DuckhuntTest(unittest.TestCase):
    
    def setUp(self):
        self.bom = Bom()
        self.bom_populated = self.populate_bom()

    def populate_bom(self):
        b = Bom()
        p = Passenger()
        p.a_str = 'Pax Thomaç'

        b.a_str = '2344-cece-455'
        b.a_list = ['abc', 123, p]
        b.a_dict = dict()
        b.a_dict['key'] = 'value'
        b.a_tuple = (1, 2)
        b.a_object = datetime(1982, 1, 30  , 8, 30)
        b.a_duckhunt = p

        return b

    def test_not_set_attr(self):
        self.assertEqual(self.bom.a_str, None)
        self.assertEqual(self.bom.a_int, None)
        self.assertEqual(self.bom.a_list, None)
        self.assertEqual(self.bom.a_dict, None)
        self.assertEqual(self.bom.a_tuple, None)
        self.assertEqual(self.bom.a_object, None)
        self.assertEqual(self.bom.a_duckhunt, None)

    def test_wrong_types(self):
        with self.assertRaises(TypeError):
            self.bom.a_str = 0
        with self.assertRaises(TypeError):
            self.bom.a_int = ''
        with self.assertRaises(TypeError):
            self.bom.a_list = dict
        with self.assertRaises(TypeError):
            self.bom.a_dict = list()
        with self.assertRaises(TypeError):
            self.bom.a_tuple = 0
        with self.assertRaises(TypeError):
            self.bom.a_object = 0
        with self.assertRaises(TypeError):
            self.bom.a_duckhunt = 0

    def test_attribute_name_get(self):
        with self.assertRaises(AttributeError):
            self.bom.xxx

    def test_attribute_name_set(self):
        with self.assertRaises(AttributeError):
            self.bom.xxx = 0

    def test_pickle(self):
        ser = pickle.dumps(self.bom_populated)
        des_bom = pickle.loads(ser)
        ser2 = pickle.dumps(des_bom)
        self.assertEqual(ser, ser2)

    def test_stringify(self):
        print(self.bom_populated)
        bom_str = str(self.bom_populated)
        self.assertTrue('a_object' in bom_str)
        json.loads(bom_str)


if __name__ == '__main__':
    unittest.main()



from functools import wraps
import jsonpickle

_attr_prefix = "__"

def json_str(cls):
    c_str = jsonpickle.encode(cls, unpicklable=False)
    return c_str.replace('\"'+_attr_prefix, '\"')


def getter_setter_gen(name, type_):
    def getter(self):
        try:
            return getattr(self, _attr_prefix + name)
        except AttributeError:
            return None

    def setter(self, value):
        
        if not isinstance(value, type_):
            raise TypeError(
                "%s attribute must be set to an instance of %s" % (name, type_))

        setattr(self, _attr_prefix + name, value)
    return property(getter, setter)


def frozen_setattr(cls, key, value):
    try:
        cls.__frozen_attrs[key](cls, key, value)
    except KeyError:
        raise AttributeError("%s attribute has not been declared" % (key))

def _duckhunt(cls, allow_new_attr):
    cls_dct = dict()
    frozen_attrs = dict()
    for key, value in cls.__dict__.items():
        # print key, value
        if isinstance(value, type):
            # print '  ',value
            value = getter_setter_gen(key, value)
        cls_dct[key] = value
        frozen_attrs[key] = object.__setattr__
        frozen_attrs[_attr_prefix + key] = object.__setattr__

    # remove __dict__ key 0> fix pickle issue
    del cls_dct['__dict__']

    # Creates a new class, using the modified dictionary as the class dict:
    n = type(cls)(cls.__name__, cls.__bases__, cls_dct)
    if not allow_new_attr:
        n.__setattr__ = frozen_setattr
        n.__frozen_attrs = frozen_attrs
    n.__str__ = json_str
    
    return n    

def duckhunt(cls):
    return _duckhunt(cls, False)

def duckhuntlight(cls):
    return _duckhunt(cls, True)


from distutils.core import setup


setup(
  name = 'duckhunt',
  packages = ['duckhunt'], # this must be the same as the name above
  version = '0.1',
  install_requires = ['jsonpickle>=0.9.5'],
  description = 'Library to  frozen class attribute and check types',
  author = 'Vincenzo Marafioti',
  author_email = 'enzo.mar@gmail.com',
  url = 'https://vinmar@bitbucket.org/vinmar/duckhunt.git', # use the URL to the github repo
  keywords = ['bom', 'class', 'setter', 'getter'], # arbitrary keywords
  classifiers = [],
)




#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest
from duckhunt import *
from datetime import datetime
import pickle
import json
import time                                                

def timeit(method):

    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()

        print ('{:10} {:20} {:f} [ms]'.format(
              args, method.__name__, (te-ts)*1000))
        return result

    return timed

@duckhunt
class PassengerD(object):
    a_str = str
    a_int = int
    a_list = list
    a_dict = dict
    a_tuple = tuple
    a_object = datetime


@duckhunt
class BomD(object):
    a_str = str
    a_int = int
    a_list = list
    a_dict = dict
    a_tuple = tuple
    a_object = datetime
    a_duckhunt = PassengerD

@duckhuntlight
class PassengerL(object):
    a_str = str
    a_int = int
    a_list = list
    a_dict = dict
    a_tuple = tuple
    a_object = datetime


@duckhuntlight
class BomL(object):
    a_str = str
    a_int = int
    a_list = list
    a_dict = dict
    a_tuple = tuple
    a_object = datetime
    a_duckhunt = PassengerL

class Passenger(object):
    def __init__(self):
        self.a_str = str()
        self.a_int = int()
        self.a_list = list()
        self.a_dict = dict()
        self.a_tuple = tuple()
        self.a_object = datetime


class Bom(object):
    def __init__(self):
        self.a_str = str()
        self.a_int = int()
        self.a_list = list()
        self.a_dict = dict()
        self.a_tuple = tuple()
        self.a_object = datetime
        self.a_duckhunt = Passenger()


@timeit
def run_class(iterations):
    for i in xrange(iterations):
        p = Passenger()
        b = Bom()
        p.a_str = 'Pax Thomaç'

        b.a_str = '2344-cece-455'
        b.a_list = ['abc', 123, p]
        b.a_dict['key'] = 'value'
        b.a_tuple = (1, 2)
        b.a_object = datetime(1982, 1, 30  , 8, 30)
        b.a_duckhunt = p

@timeit
def run_duckthunt(iterations):
    for i in xrange(iterations):
        p = PassengerD()
        b = BomD()
        p.a_str = 'Pax Thomaç'

        b.a_str = '2344-cece-455'
        b.a_list = ['abc', 123, p]
        b.a_dict = dict()
        b.a_dict['key'] = 'value'
        b.a_tuple = (1, 2)
        b.a_object = datetime(1982, 1, 30  , 8, 30)
        b.a_duckhunt = p


@timeit
def run_duckthuntlight(iterations):
    for i in xrange(iterations):
        p = PassengerL()
        b = BomL()
        p.a_str = 'Pax Thomaç'

        b.a_str = '2344-cece-455'
        b.a_list = ['abc', 123, p]
        b.a_dict = dict()
        b.a_dict['key'] = 'value'
        b.a_tuple = (1, 2)
        b.a_object = datetime(1982, 1, 30  , 8, 30)
        b.a_duckhunt = p


def run(iterations):
    run_class(iterations)
    run_duckthuntlight(iterations)
    run_duckthunt(iterations)

run(10)
print('-------')
run(100)
print('-------')
run(200)
print('-------')
run(1000)
print('-------')
run(10000)




